function mosaic = sift_triangulation_example(im1, im2)
% SIFT_TRIANGULATION_EXAMPLE
% 
% Example of using different multiview geometry scripts, SIFT and RANSAC
% to triangulate feature points in two uncalibrated images and estimate
% geometry (i.e., structure) relative to one camera. 

% Note that your results may vary according to input images, but in general
% should provide some sensible result. If you have gross outliers in your
% reconstruction, zoom in on the biggest cluster and your cameras should
% be located somewehere nearby.

focal_length = 719.5459; %Pretty random choice. If you know it for your camera, set it!



 s = RandStream('mcg16807','Seed',10); % fixed random seed for debugging
 RandStream.setGlobalStream(s);

 
 
if nargin == 0
  im1 = imread(fullfile('001.jpg')) ;
  im2 = imread(fullfile('002.jpg')) ;
end

% make single
im1 = im2single(im1) ;
im2 = im2single(im2) ;

% make grayscale
if size(im1,3) > 1, im1g = rgb2gray(im1) ; else im1g = im1 ; end
if size(im2,3) > 1, im2g = rgb2gray(im2) ; else im2g = im2 ; end

% --------------------------------------------------------------------
%                                                         SIFT matches
% --------------------------------------------------------------------

[f1,d1] = vl_sift(im1g) ;
[f2,d2] = vl_sift(im2g) ;

f1 = f1([2,1],:);
f2 = f2([2,1],:);

[matches(1,:), matches(2,:)] = matchSIFTdesImagesBidirectional(d1, d2);
f1 = f1([2 1],matches(1,:));
f2 = f2([2 1],matches(2,:));

% translate to center and invert the vertical
f1(1,:) = -(f1(1,:) - size(im1g,2)/2);
f1(2,:) = -(f1(2,:) - size(im1g,1)/2);

f2(1,:) = -(f2(1,:) - size(im2g,2)/2);
f2(2,:) = -(f2(2,:) - size(im2g,1)/2);


% --------------------------------------------------------------------
%                                                   Fundamental matrix
% --------------------------------------------------------------------


% Assemble homogeneous feature coordinates for fitting of the
% fundamental matrix, note that [x,y] corresponds to [col, row]
%     


t = .002;  % Distance threshold for deciding outliers
     [F, inliers] = ransacfitfundmatrix(f1,f2, t,1);
     
fprintf('%d inliers / %d SIFT matches = %.2f%%\n', ...
    length(inliers), size(f1,2), 100*length(inliers)/size(f1,2));

 
     % Display both images overlayed with inlying matched feature points
     show(double(im1)+double(im2),4), set(4,'name','Inlying matches'), hold on    
     plot(size(im1g,2)/2-f1(1,inliers),size(im1g,1)/2-f1(2,inliers),'r+');
     plot(size(im2g,2)/2-f2(1,inliers),size(im2g,1)/2-f2(2,inliers),'g+');    
      
     for n = inliers
 	line([size(im1g,2)/2-f1(1,n) size(im1g,2)/2-f2(1,n)],...
        [size(im1g,1)/2-f1(2,n) size(im1g,1)/2-f2(2,n)],'color',[0 0 1])
     end
 
     % Sanity check, epipolar lines should look reasonable   
     % Click mouse in one image, consider epipolar line in the other.
     fig=vgg_gui_F(im1,im2,F)
     
% --------------------------------------------------------------------
%                                                 Find camera matrices
% --------------------------------------------------------------------
imsize = [[size(im1,1);size(im1,2)],[size(im2,1);size(im2,2)]];
K = eye(3);
K(1,1) = focal_length;
K(2,2) = focal_length;

% Essential Matrix
    E = K' * F * K; % MVG Page 257 Equation 9.12

    % Decompose Essential Matrix
    [R1, R2, t1, t2] = PoseEMat(E); % MVG Page 257-259
    
    % Four possible solution
    Rt(:,:,1) =[R1 t1];
    Rt(:,:,2) =[R1 t2];
    Rt(:,:,3) =[R2 t1];
    Rt(:,:,4) =[R2 t2];
    
    % triangulation 
    P{1} = K * [eye(3) [0;0;0]];
    for i=1:4
        clear X;
        P{2} = K * Rt(:,:,i);
         
        for j=inliers
            u = [f1(1:2,j),f2(1:2,j)];
            X(:,j) = vgg_X_from_xP_nonlin(u,P,repmat([imsize(2,1);imsize(1,1)],1,2));
        end
        X = X(1:3,:) ./ X([4 4 4],:);
        
        dprd = Rt(3,1:3,i) * ((X(:,:) - repmat(Rt(1:3,4,i),1,size(X,2))));
        goodCnt(i) = sum(X(3,:)>0 & dprd > 0);
    end
    
    % pick one solution from the four
    goodCnt
    [~, bestIndex]=max(goodCnt);
    Rtbest = Rt(:,:,bestIndex);
    i= bestIndex;
    clear X;
    P{2} = K * Rtbest;%%t(:,:,i);
% --------------------------------------------------------------------
%                                    Triangulate using camera matrices 
% --------------------------------------------------------------------

    for j=inliers
            u = [f1(1:2,j),f2(1:2,j)];
        X(:,j) = vgg_X_from_xP_nonlin(u,P,repmat([imsize(2,1);imsize(1,1)],1,2));
    end
    X = X(1:3,:) ./ X([4 4 4],:);
    

% --------------------------------------------------------------------
%                                               Two view visualization
% --------------------------------------------------------------------

% two view visualization
figure
plot3(X(1,:),X(2,:),X(3,:),'.')
axis equal
title(['two views, inliers: ' num2str(nnz(inliers))]);
drawCamera(inverseCameraRt([eye(3) [0;0;0]]), imsize(2,1), imsize(1,1), K(1,1), 0.001,1);
drawCamera(inverseCameraRt(Rt(:,:,i)), imsize(2,2), imsize(1,2), K(1,1), 0.001,3);
axis tight




end